@@ -10,7 +10,7 @@ import asyncssh
 from prompt_toolkit.application.current import AppSession, create_app_session
 from prompt_toolkit.data_structures import Size
 from prompt_toolkit.eventloop import get_event_loop
-from prompt_toolkit.input import create_pipe_input
+from prompt_toolkit.input import PipeInput, create_pipe_input
 from prompt_toolkit.output.vt100 import Vt100_Output
 
 __all__ = ["PromptToolkitSSHSession", "PromptToolkitSSHServer"]
@@ -18,9 +18,13 @@ __all__ = ["PromptToolkitSSHSession", "PromptToolkitSSHServer"]
 
 class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
     def __init__(
-        self, interact: Callable[["PromptToolkitSSHSession"], Awaitable[None]]
+        self,
+        interact: Callable[["PromptToolkitSSHSession"], Awaitable[None]],
+        *,
+        enable_cpr: bool,
     ) -> None:
         self.interact = interact
+        self.enable_cpr = enable_cpr
         self.interact_task: Optional[asyncio.Task[None]] = None
         self._chan: Optional[Any] = None
         self.app_session: Optional[AppSession] = None
@@ -28,7 +32,7 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
         # PipInput object, for sending input in the CLI.
         # (This is something that we can use in the prompt_toolkit event loop,
         # but still write date in manually.)
-        self._input = create_pipe_input()
+        self._input: Optional[PipeInput] = None
         self._output: Optional[Vt100_Output] = None
 
         # Output object. Don't render to the real stdout, but write everything
@@ -86,18 +90,20 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
         term = self._chan.get_terminal_type()
 
         self._output = Vt100_Output(
-            self.stdout, self._get_size, term=term, write_binary=False
+            self.stdout, self._get_size, term=term, enable_cpr=self.enable_cpr
         )
-        with create_app_session(input=self._input, output=self._output) as session:
-            self.app_session = session
-            try:
-                await self.interact(self)
-            except BaseException:
-                traceback.print_exc()
-            finally:
-                # Close the connection.
-                self._chan.close()
-                self._input.close()
+
+        with create_pipe_input() as self._input:
+            with create_app_session(input=self._input, output=self._output) as session:
+                self.app_session = session
+                try:
+                    await self.interact(self)
+                except BaseException:
+                    traceback.print_exc()
+                finally:
+                    # Close the connection.
+                    self._chan.close()
+                    self._input.close()
 
     def terminal_size_changed(
         self, width: int, height: int, pixwidth: object, pixheight: object
@@ -107,6 +113,10 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
             self.app_session.app._on_resize()
 
     def data_received(self, data: str, datatype: object) -> None:
+        if self._input is None:
+            # Should not happen.
+            return
+
         self._input.send_text(data)
 
 
@@ -141,16 +151,25 @@ class PromptToolkitSSHServer(asyncssh.SSHServer):  # type: ignore
             )
         )
         loop.run_forever()
+
+    :param enable_cpr: When `True`, the default, try to detect whether the SSH
+        client runs in a terminal that responds to "cursor position requests".
+        That way, we can properly determine how much space there is available
+        for the UI (especially for drop down menus) to render.
     """
 
     def __init__(
-        self, interact: Callable[[PromptToolkitSSHSession], Awaitable[None]]
+        self,
+        interact: Callable[[PromptToolkitSSHSession], Awaitable[None]],
+        *,
+        enable_cpr: bool = True,
     ) -> None:
         self.interact = interact
+        self.enable_cpr = enable_cpr
 
     def begin_auth(self, username: str) -> bool:
         # No authentication.
         return False
 
     def session_requested(self) -> PromptToolkitSSHSession:
-        return PromptToolkitSSHSession(self.interact)
+        return PromptToolkitSSHSession(self.interact, enable_cpr=self.enable_cpr)
