@@ -61,6 +61,7 @@ class Char:
         "\x1b": "^[",  # Escape
         "\x1c": "^\\",
         "\x1d": "^]",
+        "\x1e": "^^",
         "\x1f": "^_",
         "\x7f": "^?",  # ASCII Delete (backspace).
         # Special characters. All visualized like Vim does.
@@ -102,7 +103,7 @@ class Char:
         "\xa0": " ",
     }
 
-    def __init__(self, char: str = " ", style: str = ""):
+    def __init__(self, char: str = " ", style: str = "") -> None:
         # If this character has to be displayed otherwise, take that one.
         if char in self.display_mappings:
             if char == "\xa0":
@@ -119,16 +120,23 @@ class Char:
         # as a member for performance.)
         self.width = get_cwidth(char)
 
-    def __eq__(self, other) -> bool:
+    # In theory, `other` can be any type of object, but because of performance
+    # we don't want to do an `isinstance` check every time. We assume "other"
+    # is always a "Char".
+    def _equal(self, other: "Char") -> bool:
         return self.char == other.char and self.style == other.style
 
-    def __ne__(self, other) -> bool:
+    def _not_equal(self, other: "Char") -> bool:
         # Not equal: We don't do `not char.__eq__` here, because of the
         # performance of calling yet another function.
         return self.char != other.char or self.style != other.style
 
+    if not TYPE_CHECKING:
+        __eq__ = _equal
+        __ne__ = _not_equal
+
     def __repr__(self) -> str:
-        return "%s(%r, %r)" % (self.__class__.__name__, self.char, self.style)
+        return f"{self.__class__.__name__}({self.char!r}, {self.style!r})"
 
 
 _CHAR_CACHE: FastDictCache[Tuple[str, str], Char] = FastDictCache(
@@ -311,7 +319,7 @@ class WritePosition:
         self.height = height
 
     def __repr__(self) -> str:
-        return "%s(x=%r, y=%r, width=%r, height=%r)" % (
+        return "{}(x={!r}, y={!r}, width={!r}, height={!r})".format(
             self.__class__.__name__,
             self.xpos,
             self.ypos,
